@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2000 - 2004 Christian Kreibich <christian@whoop.org>.
+Copyright (C) 2000 - 2006 Christian Kreibich <christian@whoop.org>.
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -45,6 +45,12 @@ struct lnd_packet
 
   guchar               *data;
 
+  /* The allocated size of the data chunk. The packet may actually
+   * be using less and can never use more; see ph.caplen for the
+   * actually used size.
+   */
+  guint                 data_size;
+
   LND_TracePart        *part;
 
   /* Protocols at various layers into the packet */
@@ -75,16 +81,22 @@ struct lnd_packet
  * that can be reported to observers using libnd_packet_set_observer().
  */
 typedef enum {
-  LND_PACKET_INITIALIZED = (1 << 0), /* Packet got initialized */
-  LND_PACKET_MODIFIED    = (1 << 1), /* Packet got modified */
-  LND_PACKET_DELETE_PRE  = (1 << 2), /* Packet gets deleted (pre-action) */
-  LND_PACKET_DELETE_POST = (1 << 3), /* Packet got deleted (post-action) */
-  LND_PACKET_INSERT_PRE  = (1 << 4), /* Packet gets inserted (pre-action) */
-  LND_PACKET_INSERT_POST = (1 << 5), /* Packet got inserted (post-action) */
-  LND_PACKET_DUPLICATED  = (1 << 6), /* Packet got duplicated */
-  LND_PACKET_VISIBILITY  = (1 << 7), /* Packet visibility changed */
-  LND_PACKET_UPDATED     = (1 << 8), /* Packet got updated (partial initialization) */
-  LND_PACKET_FIXED       = (1 << 9)  /* Packet checksums got fixed */
+  LND_PACKET_INITIALIZED  = (1 << 0),    /* Packet got initialized */
+  LND_PACKET_MODIFIED     = (1 << 1),    /* Packet got modified */
+  LND_PACKET_DELETE_PRE   = (1 << 2),    /* Packet gets deleted (pre-action) */
+  LND_PACKET_DELETE_POST  = (1 << 3),    /* Packet got deleted (post-action) */
+  LND_PACKET_INSERT_PRE   = (1 << 4),    /* Packet gets inserted (pre-action) */
+  LND_PACKET_INSERT_POST  = (1 << 5),    /* Packet got inserted (post-action) */
+  LND_PACKET_DUPLICATED   = (1 << 6),    /* Packet got duplicated */
+  LND_PACKET_VISIBILITY   = (1 << 7),    /* Packet visibility changed */
+  LND_PACKET_UPDATED      = (1 << 8),    /* Packet got updated (partial initialization) */
+  LND_PACKET_FIXED        = (1 << 9),    /* Packet checksums got fixed */
+  LND_PACKET_LEN_CHANGED  = (1 << 10),   /* Packet's wire size changed */
+  LND_PACKET_CAPLEN_CHANGED = (1 << 11), /* Packet's caplen changed */
+  
+  /* If you add something here, also add a callback invocation in
+   * libnd_packet_tell_observers().
+   */
 } LND_PacketObserverOp;
 
 
@@ -100,6 +112,8 @@ typedef struct lnd_packet_observer
   void  (*packet_visibility) (LND_Packet *packet);
   void  (*packet_updated) (LND_Packet *packet);
   void  (*packet_fixed) (LND_Packet *packet);
+  void  (*packet_len_changed) (LND_Packet *packet);
+  void  (*packet_caplen_changed) (LND_Packet *packet);
 
 } LND_PacketObserver;
 
@@ -112,10 +126,12 @@ typedef struct lnd_packet_observer
  * Functions of this signature are used to iterate over the
  * protocols that are contained in packtes. See
  * libnd_packet_foreach_proto() and libnd_packet_foreach_proto_backward().
+ *
+ * Returns: %TRUE if the iteration is to continue, %FALSE on abort.
  */
-typedef void (*LND_PacketFunc)(LND_Packet     *packet,
-			       LND_ProtoData  *pd,
-			       void           *user_data);
+typedef gboolean (*LND_PacketFunc)(LND_Packet     *packet,
+				   LND_ProtoData  *pd,
+				   void           *user_data);
 
 
 /**
@@ -209,7 +225,8 @@ void            libnd_packet_set_data(LND_Packet *packet,
  *
  * Returns a deep copy of a packet, but with the
  * list pointers set to NULL (i.e. the copy does not
- * belong to any packet list).
+ * belong to any packet list). It also doesn't belong
+ * to any trace part (i.e., packet->part == NULL).
  *
  * Returns: packet copy or NULL when error occured.
  */
@@ -299,6 +316,13 @@ gboolean        libnd_packet_is_complete(const LND_Packet *packet);
  */
 void            libnd_packet_update(LND_Packet *packet, LND_Protocol *proto, guint nesting);
 
+/**
+ * libnd_packet_get_num_protos - returns number of protocols in packet.
+ * @packet: packet to query.
+ *
+ * Returns: the number of protocols found in @packet.
+ */
+guint           libnd_packet_get_num_protos(LND_Packet *packet);
 
 /**
  * libnd_packet_get_data - data accessing function
@@ -529,6 +553,32 @@ int             libnd_packet_get_proto_nesting(const LND_Packet *packet,
 					       const LND_Protocol *proto,
 					       guchar *data);
 
+
+/**
+ * libnd_packet_get_proto_index - index of a protocol in the packet.
+ * @packet: packet to query.
+ *
+ * Returns: the index of the given protocol instance in the packet,
+ * starting from zero. If the instance cannot be located in the packet,
+ * a negative value is returned. For example, if a packet contains
+ * Ethernet, IP, and TCP, then those protocols would be at indices
+ * 0, 1, and 2, respectively. This is an O(n) operation.
+ */
+int             libnd_packet_get_proto_index(const LND_Packet *packet,
+					     const LND_ProtoInst *pi);
+
+
+/**
+ * libnd_packet_get_nth_proto - returns nth protocol in packet.
+ * @packet: packet to query.
+ *
+ * Returns the LND_ProtoData structure corresponding to the nth
+ * protocol in the packet, or %NULL if no nth protocol exists.
+ * Indexing starts at 0. This is an O(n) operation.
+ */
+LND_ProtoData  *libnd_packet_get_nth_proto(LND_Packet *packet, int n);
+
+
 /**
  * libnd_packet_fix - calls fix_packet() callback for each protocol.
  * @packet: packet to fix.
@@ -541,6 +591,50 @@ int             libnd_packet_get_proto_nesting(const LND_Packet *packet,
  */
 gboolean        libnd_packet_fix(LND_Packet *packet);
 
+/**
+ * libnd_packet_get_last_fixable_proto - offsert of last fixable protocol in packet.
+ * @packet: packet to query.
+ *
+ * Returns: the index (starting from 0) of the last protocol in the
+ * given packet at which libnd_packet_fix() will do anything.
+ */
+int             libnd_packet_get_last_fixable_proto(const LND_Packet *packet);
+
+/**
+ * libnd_packet_adjust_len - changes a packet's wire size.
+ * @packet: packet to adjust.
+ * @delta: amount of size change, positive or negative.
+ *
+ * The function adjusts the packet's wire size and gives the protocols
+ * in the packet a chance to accomodate the size change. Note: the size
+ * change refers to the size of the packet as seen on the wire, not the
+ * snaplen used by pcap: a size change of -10 on a packet of 1500 bytes
+ * of which you've captured only 68 will leave 68 unmodified, but change
+ * the packet's contents as captures as if the original size had been
+ * 1490 bytes.
+ * 
+ * If you want to change the caplen, use libnd_packet_adjust_caplen(). 
+ *
+ * Returns: %TRUE if the change succeded, %FALSE if not.
+ */
+gboolean         libnd_packet_adjust_len(LND_Packet *packet, int delta);
+
+/**
+ * libnd_packet_adjust_caplen - changes a packet's wire size.
+ * @packet: packet to adjust.
+ * @delta: amount of size change, positive or negative.
+ *
+ * The function adjusts the size of the part of the packet that has been
+ * captured. With positive delta's, the new part of the packet is
+ * initialized to zero. If the caplen would grow beyond the wire length,
+ * the latter is updated accordingly.
+ * 
+ * If you want to change (only) the wire length, use
+ * libnd_packet_adjust_len().
+ *
+ * Returns: %TRUE if the change succeded, %FALSE if not.
+ */
+gboolean            libnd_packet_adjust_caplen(LND_Packet *packet, int delta);
 
 /**
  * libnd_packet_observer_new - creates new packet observer.
