@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2000 - 2004 Christian Kreibich <christian@whoop.org>.
+Copyright (C) 2000 - 2006 Christian Kreibich <christian@whoop.org>.
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -28,6 +28,6 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 /**
  * LIBND_LAST_UPDATE - timestamp of last CVS update of the codebase
  */
-#define LIBND_LAST_UPDATE "Wed May  4 18:18:24 BST 2005"
+#define LIBND_LAST_UPDATE "Sun Jun 24 13:42:37 PDT 2007"
 
 #endif
